@@ -1,12 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-Table-Group.c: Implements the grouping objects for elements on a table
+/* 
+ * e-table-group-container.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
+ * Authors:
  *   Chris Lahey <clahey@ximian.com>
- *   Miguel de Icaza (miguel@gnu.org)
  *
- * Copyright 1999, 2000 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -148,13 +160,10 @@ e_table_group_container_construct (GnomeCanvasGroup *parent, ETableGroupContaine
 	etgc->n = n;
 	etgc->ascending = column.ascending;
 
-	
-	etgc->font = gdk_font_load ("lucidasans-10");
-	if (!etgc->font){
-		etgc->font = GTK_WIDGET (GNOME_CANVAS_ITEM (etgc)->canvas)->style->font;
-		
-		gdk_font_ref (etgc->font);
-	}
+	etgc->font = GTK_WIDGET (GNOME_CANVAS_ITEM (etgc)->canvas)->style->font;
+
+	gdk_font_ref (etgc->font);
+
 	etgc->open = TRUE;
 }
 
@@ -333,26 +342,20 @@ etgc_unrealize (GnomeCanvasItem *item)
 static void
 compute_text (ETableGroupContainer *etgc, ETableGroupContainerChildNode *child_node)
 {
-	gchar *text, *s1, *s2;
+	gchar *text;
 
 	if (etgc->ecol->text) {
-		s1 = e_utf8_to_locale_string (etgc->ecol->text);
-		s2 = e_utf8_to_locale_string (child_node->string);
 		text = g_strdup_printf ((child_node->count == 1)
 					? _("%s : %s (%d item)")
 					: _("%s : %s (%d items)"),
-					s1, s2,
+					etgc->ecol->text, child_node->string,
 					(gint) child_node->count);
-		g_free (s1);
-		g_free (s2);
 	} else {
-		s1 = e_utf8_to_locale_string (child_node->string);
 		text = g_strdup_printf ((child_node->count == 1)
 					? _("%s (%d item)")
 					: _("%s (%d items)"),
-					s1,
+					child_node->string,
 					(gint) child_node->count);
-		g_free (s1);
 	}
 	gnome_canvas_item_set (child_node->text, 
 			       "text", text,
