@@ -1,16 +1,30 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-item.c: A GnomeCanvasItem that is a view of an ETableModel.
- *
- * Author:
- *   Christopher James Lahey <clahey@ximian.com>
- *   Miguel de Icaza (miguel@gnu.org)
- *
+/* 
+ * e-table-item.c
  * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza <miguel@gnu.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
+/*
  * TODO:
  *   Add a border to the thing, so that focusing works properly.
- *
  */
 #include <config.h>
 
@@ -36,6 +50,12 @@
 
 #define d(x)
 
+#if d(!)0
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)), g_print ("%s: e_table_item_leave_edit\n", __FUNCTION__))
+#else
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)))
+#endif
+
 /* FIXME: Do an analysis of which cell functions are needed before
    realize and make sure that all of them are doable by all the cells
    and that all of the others are only done after realization. */
@@ -146,9 +166,45 @@ view_to_model_col(ETableItem *eti, int col)
 	return ecol ? ecol->col_idx : -1;
 }
 
+inline static void
+eti_grab (ETableItem *eti, guint32 time)
+{
+	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (eti);
+	d(g_print ("%s: time: %d\n", __FUNCTION__, time));
+	if (eti->grabbed_count == 0) {
+		eti->gtk_grabbed = FALSE;
+		if (!gnome_canvas_item_grab(item,
+					    GDK_BUTTON1_MOTION_MASK | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK
+					    | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK,
+					    NULL, time)) {
+			d(g_print ("%s: gtk_grab_add\n", __FUNCTION__));
+			gtk_grab_add (GTK_WIDGET (item->canvas));
+			eti->gtk_grabbed = TRUE;
+		}
+	}
+	eti->grabbed_count ++;
+}
+
+inline static void
+eti_ungrab (ETableItem *eti, guint32 time)
+{
+	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (eti);
+	d(g_print ("%s: time: %d\n", __FUNCTION__, time));
+	eti->grabbed_count --;
+	if (eti->grabbed_count == 0) {
+		if (eti->gtk_grabbed) {
+			d(g_print ("%s: gtk_grab_remove\n", __FUNCTION__));
+			gtk_grab_remove (GTK_WIDGET (item->canvas));
+		}
+		gnome_canvas_item_ungrab(item, time);
+	}
+}
+
 inline static gboolean
 eti_editing (ETableItem *eti)
 {
+	d(g_print("%s: %s\n", __FUNCTION__, (eti->editing_col == -1) ? "false":"true"));
+	
 	if (eti->editing_col == -1)
 		return FALSE;
 	else
@@ -156,13 +212,13 @@ eti_editing (ETableItem *eti)
 }
 
 inline static GdkColor *
-eti_get_cell_background_color (ETableItem *eti, int row, int col, gboolean selected, gboolean *allocated)
+eti_get_cell_background_color (ETableItem *eti, int row, int col, gboolean selected, gboolean *allocatedp)
 {
+  	ECellView *ecell_view = eti->cell_views [col];
 	GtkWidget *canvas = GTK_WIDGET(GNOME_CANVAS_ITEM(eti)->canvas);
-	GdkColor *background;
-
-	if (allocated)
-		*allocated = FALSE;
+	GdkColor *background, bg;
+	gchar *color_spec = NULL;
+	gboolean allocated = FALSE;
 
 	if (selected){
 		if (GTK_WIDGET_HAS_FOCUS(canvas))
@@ -173,17 +229,29 @@ eti_get_cell_background_color (ETableItem *eti, int row, int col, gboolean selec
 		background = &canvas->style->base [GTK_STATE_NORMAL];
 	}
 
+	color_spec = e_cell_get_bg_color (ecell_view, row);
+
+	if (color_spec != NULL) {
+		if (gdk_color_parse (color_spec, &bg)) {
+			background = gdk_color_copy (&bg);
+			allocated = TRUE;
+		}
+	}
+
 	if (eti->alternating_row_colors) {
 		if (row % 2) {
 		
 		} else {
-			if (allocated)
-				*allocated = TRUE;
-			background = gdk_color_copy (background);
+			if (!allocated) {
+				background = gdk_color_copy (background);
+				allocated = TRUE;
+			}
 			e_hsv_tweak (background, 0.0f, 0.0f, -0.05f);
 			gdk_color_alloc (gtk_widget_get_colormap (GTK_WIDGET (canvas)), background);
 		}
 	}
+	if (allocatedp)
+		*allocatedp = allocated;
 
 	return background;
 }
@@ -389,6 +457,8 @@ eti_remove_table_model (ETableItem *eti)
 
 	gtk_signal_disconnect (GTK_OBJECT (eti->table_model),
 			       eti->table_model_pre_change_id);
+	gtk_signal_disconnect (GTK_OBJECT (eti->table_model),
+			       eti->table_model_no_change_id);
 	gtk_signal_disconnect (GTK_OBJECT (eti->table_model),
 			       eti->table_model_change_id);
 	gtk_signal_disconnect (GTK_OBJECT (eti->table_model),
@@ -404,6 +474,7 @@ eti_remove_table_model (ETableItem *eti)
 		gtk_object_unref (GTK_OBJECT (eti->source_model));
 
 	eti->table_model_pre_change_id = 0;
+	eti->table_model_no_change_id = 0;
 	eti->table_model_change_id = 0;
 	eti->table_model_row_change_id = 0;
 	eti->table_model_cell_change_id = 0;
@@ -772,16 +843,22 @@ eti_show_cursor (ETableItem *eti, int delay)
 	
 	if (!((GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED) && eti->cell_views_realized))
 		return;
+
+	if (eti->frozen_count > 0) {
+		eti->queue_show_cursor = TRUE;
+		return;
+	}
 	
 	gtk_object_get(GTK_OBJECT(eti->selection),
 		       "cursor_row", &cursor_row,
 		       NULL);
 
+	d(g_print ("%s: cursor row: %d\n", __FUNCTION__, cursor_row));
+
 	if (cursor_row != -1) {
 		cursor_row = model_to_view_row (eti, cursor_row);
-		d(g_print ("%s: cursor row: %d\n", __FUNCTION__, cursor_row));
 		eti_request_region_show (eti,
-					 0, cursor_row, eti->cols + 1, cursor_row + 1,
+					 0, cursor_row, eti->cols - 1, cursor_row,
 					 delay);
 	}
 }
@@ -812,6 +889,10 @@ eti_check_cursor_bounds (ETableItem *eti)
 
 	if (!((GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED) && eti->cell_views_realized))
 		return;
+
+	if (eti->frozen_count > 0) {
+		return;
+	}
 	
 	gtk_object_get(GTK_OBJECT(eti->selection),
 		       "cursor_row", &cursor_row,
@@ -826,12 +907,14 @@ eti_check_cursor_bounds (ETableItem *eti)
 		return;
 	}
 
+	d(g_print ("%s: model cursor row: %d\n", __FUNCTION__, cursor_row));
+
 	cursor_row = model_to_view_row (eti, cursor_row);
 
 	d(g_print ("%s: cursor row: %d\n", __FUNCTION__, cursor_row));
 
 	eti_get_region (eti,
-			0, cursor_row, eti->cols + 1, cursor_row + 1,
+			0, cursor_row, eti->cols - 1, cursor_row,
 			&x1, &y1, &x2, &y2);
 	eti->cursor_x1 = x1;
 	eti->cursor_y1 = y1;
@@ -852,11 +935,12 @@ eti_maybe_show_cursor(ETableItem *eti, int delay)
 }
 
 static gboolean
-eti_idle_maybe_show_cursor_cb (gpointer data)
+eti_idle_show_cursor_cb (gpointer data)
 {
 	ETableItem *eti = data;
 	if (!GTK_OBJECT_DESTROYED (eti)) {
-		eti_maybe_show_cursor (eti, 0);
+		eti_show_cursor (eti, 0);
+		eti_check_cursor_bounds (eti);
 	}
 	gtk_object_unref (GTK_OBJECT (eti));
 	return FALSE;
@@ -865,34 +949,83 @@ eti_idle_maybe_show_cursor_cb (gpointer data)
 static void
 eti_idle_maybe_show_cursor(ETableItem *eti)
 {
-	gtk_object_ref (GTK_OBJECT (eti));
-	g_idle_add (eti_idle_maybe_show_cursor_cb, eti);
+	d(g_print ("%s: cursor on screen: %s\n", __FUNCTION__, eti->cursor_on_screen ? "TRUE" : "FALSE"));
+	if (eti->cursor_on_screen) {
+		gtk_object_ref (GTK_OBJECT (eti));
+		g_idle_add (eti_idle_show_cursor_cb, eti);
+	}
+}
+
+static void
+eti_cancel_drag_due_to_model_change (ETableItem *eti)
+{
+	if (eti->maybe_in_drag) {
+		eti->maybe_in_drag = FALSE;
+		if (!eti->maybe_did_something)
+			e_selection_model_do_something(E_SELECTION_MODEL (eti->selection), eti->drag_row, eti->drag_col, eti->drag_state);
+	}
+	if (eti->in_drag) {
+		eti->in_drag = FALSE;
+	}
+}
+
+static void
+eti_freeze (ETableItem *eti)
+{
+	eti->frozen_count ++;
+}
+
+static void
+eti_unfreeze (ETableItem *eti)
+{
+	eti->frozen_count --;
+	if (eti->frozen_count == 0 && eti->queue_show_cursor) {
+		eti_show_cursor (eti, 0);
+		eti_check_cursor_bounds (eti);
+		eti->queue_show_cursor = FALSE;
+	}
 }
 
 /*
- * Callback routine: invoked before the ETableModel has suffers a change
+ * Callback routine: invoked before the ETableModel suffers a change
  */
 static void
 eti_table_model_pre_change (ETableModel *table_model, ETableItem *eti)
 {
+	eti_cancel_drag_due_to_model_change (eti);
 	eti_check_cursor_bounds (eti);
 	if (eti_editing (eti))
-		e_table_item_leave_edit (eti);
+		e_table_item_leave_edit_(eti);
+	eti_freeze (eti);
+}
+
+/*
+ * Callback routine: invoked when the ETableModel has not suffered a change
+ */
+static void
+eti_table_model_no_change (ETableModel *table_model, ETableItem *eti)
+{
+	eti_unfreeze (eti);
 }
 
 /*
  * Callback routine: invoked when the ETableModel has suffered a change
  */
+
 static void
 eti_table_model_changed (ETableModel *table_model, ETableItem *eti)
 {
-	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
+ 	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED)) {
+		eti_unfreeze (eti);
 		return;
+	}
 
 	eti->rows = e_table_model_row_count (eti->table_model);
 
 	free_height_cache(eti);
 
+	eti_unfreeze (eti);
+
 	eti->needs_compute_height = 1;
 	e_canvas_item_request_reflow (GNOME_CANVAS_ITEM (eti));
 	eti->needs_redraw = 1;
@@ -904,36 +1037,46 @@ eti_table_model_changed (ETableModel *table_model, ETableItem *eti)
 static void
 eti_table_model_row_changed (ETableModel *table_model, int row, ETableItem *eti)
 {
-	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
+ 	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED)) {
+		eti_unfreeze (eti);
 		return;
+	}
 
 	if ((!eti->uniform_row_height) && eti->height_cache && eti->height_cache[row] != -1 && eti_row_height_real(eti, row) != eti->height_cache[row]) {
 		eti_table_model_changed (table_model, eti);
 		return;
 	}
 
+	eti_unfreeze (eti);
+
 	eti_request_region_redraw (eti, 0, row, eti->cols - 1, row, 0);
 }
 
 static void
 eti_table_model_cell_changed (ETableModel *table_model, int col, int row, ETableItem *eti)
 {
-	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
+ 	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED)) {
+		eti_unfreeze (eti);
 		return;
+	}
 
 	if ((!eti->uniform_row_height) && eti->height_cache && eti->height_cache[row] != -1 && eti_row_height_real(eti, row) != eti->height_cache[row]) {
 		eti_table_model_changed (table_model, eti);
 		return;
 	}
 
+	eti_unfreeze (eti);
+
 	eti_request_region_redraw (eti, 0, row, eti->cols - 1, row, 0);
 }
 
 static void
 eti_table_model_rows_inserted (ETableModel *table_model, int row, int count, ETableItem *eti)
 {
-	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
+ 	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED)) {
+		eti_unfreeze (eti);
 		return;
+	}
 	eti->rows = e_table_model_row_count (eti->table_model);
 
 	if (eti->height_cache) {
@@ -944,6 +1087,8 @@ eti_table_model_rows_inserted (ETableModel *table_model, int row, int count, ETa
 			eti->height_cache[i] = -1;
 	}
 
+	eti_unfreeze (eti);
+
 	eti_idle_maybe_show_cursor(eti);
 
 	eti->needs_compute_height = 1;
@@ -955,8 +1100,10 @@ eti_table_model_rows_inserted (ETableModel *table_model, int row, int count, ETa
 static void
 eti_table_model_rows_deleted (ETableModel *table_model, int row, int count, ETableItem *eti)
 {
-	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
+ 	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED)) {
+		eti_unfreeze (eti);
 		return;
+	}
 
 	g_assert (eti->rows == -1 || row + count <= eti->rows);
 
@@ -968,6 +1115,8 @@ eti_table_model_rows_deleted (ETableModel *table_model, int row, int count, ETab
 		memmove(eti->height_cache + row, eti->height_cache + row + count, (eti->rows - row) * sizeof(int));
 	}
 
+	eti_unfreeze (eti);
+
 	eti_idle_maybe_show_cursor(eti);
 
 	eti->needs_compute_height = 1;
@@ -1026,6 +1175,10 @@ eti_add_table_model (ETableItem *eti, ETableModel *table_model)
 		GTK_OBJECT (table_model), "model_pre_change",
 		GTK_SIGNAL_FUNC (eti_table_model_pre_change), eti);
 
+	eti->table_model_no_change_id = gtk_signal_connect (
+		GTK_OBJECT (table_model), "model_no_change",
+		GTK_SIGNAL_FUNC (eti_table_model_no_change), eti);
+
 	eti->table_model_change_id = gtk_signal_connect (
 		GTK_OBJECT (table_model), "model_changed",
 		GTK_SIGNAL_FUNC (eti_table_model_changed), eti);
@@ -1356,13 +1509,16 @@ eti_init (GnomeCanvasItem *item)
 
 	eti->maybe_did_something       = TRUE;
 
-	eti->grabbed_col               = -1;
-	eti->grabbed_row               = -1;
+	eti->grabbed_count             = 0;
+	eti->gtk_grabbed               = 0;
 
 	eti->in_drag                   = 0;
 	eti->maybe_in_drag             = 0;
 	eti->grabbed                   = 0;
 
+	eti->grabbed_col               = -1;
+	eti->grabbed_row               = -1;
+
 	eti->cursor_on_screen          = FALSE;
  	eti->cursor_x1                 = -1;
 	eti->cursor_y1                 = -1;
@@ -1371,6 +1527,9 @@ eti_init (GnomeCanvasItem *item)
 
 	eti->rows                      = -1;
 
+	eti->frozen_count              = 0;
+	eti->queue_show_cursor         = FALSE;
+
 	e_canvas_item_set_reflow_callback (GNOME_CANVAS_ITEM (eti), eti_reflow);
 }
 
@@ -1448,12 +1607,13 @@ eti_unrealize (GnomeCanvasItem *item)
 	ETableItem *eti = E_TABLE_ITEM (item);
 
 	if (eti->grabbed) {
-		gtk_grab_remove (GTK_WIDGET (item->canvas));
 		eti->grabbed = FALSE;
+		d(g_print ("%s: eti_ungrab\n", __FUNCTION__));
+		eti_ungrab (eti, -1);
 	}
 
 	if (eti_editing (eti))
-		e_table_item_leave_edit (eti);
+		e_table_item_leave_edit_(eti);
 
 	gdk_gc_unref (eti->fill_gc);
 	eti->fill_gc = NULL;
@@ -1617,6 +1777,7 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 			ETableCol *ecol = e_table_header_get_column (eti->header, col);
 			ECellView *ecell_view = eti->cell_views [col];
 			gboolean col_selected = selected;
+			gboolean cursor = FALSE;
 			ECellFlags flags;
 			gboolean free_background;
 			GdkColor *background;
@@ -1624,8 +1785,10 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 			switch (eti->cursor_mode) {
 			case E_CURSOR_SIMPLE:
 			case E_CURSOR_SPREADSHEET:
-				if (cursor_col == ecol->col_idx && cursor_row == view_to_model_row(eti, row))
+				if (cursor_col == ecol->col_idx && cursor_row == view_to_model_row(eti, row)) {
 					col_selected = !col_selected;
+					cursor = TRUE;
+				}
 				break;
 			case E_CURSOR_LINE:
 				/* Nothing */
@@ -1643,6 +1806,8 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 
 			flags = col_selected ? E_CELL_SELECTED : 0;
 			flags |= GTK_WIDGET_HAS_FOCUS(canvas) ? E_CELL_FOCUSED : 0;
+			flags |= cursor ? E_CELL_CURSOR : 0;
+
 			switch (ecol->justification) {
 			case GTK_JUSTIFY_LEFT:
 				flags |= E_CELL_JUSTIFY_LEFT;
@@ -1804,7 +1969,7 @@ find_cell (ETableItem *eti, double x, double y, int *view_col_res, int *view_row
 static void
 eti_cursor_move (ETableItem *eti, gint row, gint column)
 {
-	e_table_item_leave_edit (eti);
+	e_table_item_leave_edit_(eti);
 	e_table_item_focus (eti, view_to_model_col(eti, column), view_to_model_row(eti, row), 0);
 }
 
@@ -1906,13 +2071,15 @@ eti_e_cell_event     (ETableItem *item, ECellView *ecell_view, GdkEvent *event,
 	ret_val = e_cell_event (ecell_view, event, model_col, view_col, row, flags, &actions);
 
 	if (actions & E_CELL_GRAB) {
-		gnome_canvas_item_grab(GNOME_CANVAS_ITEM(item), GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, NULL, time);
+		d(g_print ("%s: eti_grab\n", __FUNCTION__));
+		eti_grab (item, time);
 		item->grabbed_col = view_col;
 		item->grabbed_row = row;
 	}
 
 	if (actions & E_CELL_UNGRAB) {
-		gnome_canvas_item_ungrab(GNOME_CANVAS_ITEM(item), time);
+		d(g_print ("%s: eti_ungrab\n", __FUNCTION__));
+		eti_ungrab (item, time);
 		item->grabbed_col = -1;
 		item->grabbed_row = -1;
 	}
@@ -1927,6 +2094,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 	ETableItem *eti = E_TABLE_ITEM (item);
 	ECellView *ecell_view;
 	gint return_val = TRUE;
+#if d(!)0
+	gboolean leave = FALSE;
+#endif
 
 	switch (e->type){
 	case GDK_BUTTON_PRESS: {
@@ -1936,6 +2106,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		int col, row;
 		gint cursor_row, cursor_col;
 		gint new_cursor_row, new_cursor_col;
+		ECellFlags flags = 0;
+
+		d(g_print("%s: GDK_BUTTON_PRESS received, button %d\n", __FUNCTION__, e->button.button));
 
 		if (eti->tooltip->timer) {
 			gtk_timeout_remove (eti->tooltip->timer);
@@ -1953,7 +2126,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 
 			if (!find_cell (eti, realx, realy, &col, &row, &x1, &y1)) {
 				if (eti_editing (eti))
-					e_table_item_leave_edit (eti);
+					e_table_item_leave_edit_(eti);
 				return TRUE;
 			}
 
@@ -1962,7 +2135,18 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			button.x = x1;
 			button.y = y1;
 
-			return_val = eti_e_cell_event (eti, ecell_view, (GdkEvent *) &button, button.time, view_to_model_col(eti, col), col, row, 0);
+			gtk_object_get(GTK_OBJECT(eti->selection),
+				       "cursor_row", &cursor_row,
+				       "cursor_col", &cursor_col,
+				       NULL);
+
+			if (cursor_col == view_to_model_col (eti, col) && cursor_row == view_to_model_row(eti, row)) {
+				flags = E_CELL_CURSOR;
+			} else {
+				flags = 0;
+			}
+
+			return_val = eti_e_cell_event (eti, ecell_view, (GdkEvent *) &button, button.time, view_to_model_col(eti, col), col, row, flags);
 			if (return_val)
 				return TRUE;
 
@@ -2003,13 +2187,15 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 
 				if (eti_editing (eti)) {
 					return_val = eti_e_cell_event (eti, ecell_view, (GdkEvent *) &button, button.time, 
-								       view_to_model_col(eti, col), col, row, E_CELL_EDITING);
+								       view_to_model_col(eti, col), col, row, E_CELL_EDITING | E_CELL_CURSOR);
 					if (return_val)
 						return TRUE;
 				}
 			}
 
 			if (e->button.button == 1) {
+				return_val = TRUE;
+
 				eti->maybe_in_drag = TRUE;
 				eti->drag_row      = new_cursor_row;
 				eti->drag_col      = new_cursor_col;
@@ -2017,14 +2203,10 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				eti->drag_y        = realy;
 				eti->drag_state    = e->button.state;
 				eti->grabbed       = TRUE;
-				if (!gnome_canvas_item_grab(item,
-							    (1 << (4 + e->button.button)) | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK, 
-							    NULL, e->button.time))
-					gtk_grab_add (GTK_WIDGET (item->canvas));
+				d(g_print ("%s: eti_grab\n", __FUNCTION__));
+				eti_grab (eti, e->button.time);
 			}
 
-			d(g_print("Single click\n"));
-
 			break;
 		case 3:
 			e_canvas_item_grab_focus(GNOME_CANVAS_ITEM(eti), TRUE);
@@ -2053,12 +2235,15 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		int col, row;
 		gint cursor_row, cursor_col;
 
-		if (e->button.button == 1) {
-			if (eti->grabbed) {
-				gtk_grab_remove (GTK_WIDGET (item->canvas));
-				gnome_canvas_item_ungrab(item, e->button.time);
-			}
+		d(g_print("%s: GDK_BUTTON_RELEASE received, button %d\n", __FUNCTION__, e->button.button));
+
+		if (eti->grabbed) {
+			d(g_print ("%s: eti_ungrab\n", __FUNCTION__));
+			eti_ungrab (eti, e->button.time);
 			eti->grabbed = FALSE;
+		}
+
+		if (e->button.button == 1) {
 			if (eti->maybe_in_drag) {
 				eti->maybe_in_drag = FALSE;
 				if (!eti->maybe_did_something)
@@ -2079,6 +2264,13 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		case 2:
 
 			gnome_canvas_item_w2i (item, &e->button.x, &e->button.y);
+#if d(!)0
+			{
+				gboolean cell_found = find_cell (eti, e->button.x, e->button.y, &col, &row, &x1, &y1);
+				g_print("%s: find_cell(%f, %f) = %s(%d, %d, %f, %f)\n", __FUNCTION__, e->button.x, e->button.y,
+					cell_found?"true":"false", col, row, x1, y1);
+			}
+#endif
 
 			if (!find_cell (eti, e->button.x, e->button.y, &col, &row, &x1, &y1))
 				return TRUE;
@@ -2088,8 +2280,16 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				       "cursor_col", &cursor_col,
 				       NULL);
 
+
+			d(g_print("%s: GDK_BUTTON_RELEASE received, button %d, line: %d\n"
+				  "eti_editing: %s, row:%d:%d, col:%d:%d\n", __FUNCTION__, e->button.button, __LINE__,
+				  eti_editing(eti)?"true":"false", cursor_row, view_to_model_row(eti, row), cursor_col, view_to_model_col(eti, col)));
+
 			if (eti_editing (eti) && cursor_row == view_to_model_row(eti, row) && cursor_col == view_to_model_col(eti, col)){
 
+				d(g_print("%s: GDK_BUTTON_RELEASE received, button %d, line: %d\n", __FUNCTION__, e->button.button, __LINE__))
+;
+
 				ecell_view = eti->cell_views [col];
 
 				/*
@@ -2099,9 +2299,8 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				e->button.y = y1;
 
 				return_val = eti_e_cell_event (eti, ecell_view, e, e->button.time,
-							       view_to_model_col(eti, col), col, row, E_CELL_EDITING);
+							       view_to_model_col(eti, col), col, row, E_CELL_EDITING | E_CELL_CURSOR);
 			}
-			d(g_print ("Release\n"));
 			break;
 		case 3:
 			e_selection_model_right_click_up(E_SELECTION_MODEL (eti->selection));
@@ -2121,6 +2320,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 #if 0
 		double x1, y1;
 #endif
+
+		d(g_print("%s: GDK_2BUTTON_PRESS received, button %d\n", __FUNCTION__, e->button.button));
+
 		if (e->button.button == 5 ||
 		    e->button.button == 4)
 			return FALSE;
@@ -2155,7 +2357,6 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			if (model_row != -1 && model_col != -1) {
 				gtk_signal_emit (GTK_OBJECT (eti), eti_signals [DOUBLE_CLICK],
 						 model_row, model_col, e);
-				d(g_print("Double click\n"));
 			}
 		}
 		break;
@@ -2214,7 +2415,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			e->motion.y = y1;
 
 			return_val = eti_e_cell_event (eti, ecell_view, e, e->motion.time,
-						       view_to_model_col(eti, col), col, row, E_CELL_EDITING);
+						       view_to_model_col(eti, col), col, row, E_CELL_EDITING | E_CELL_CURSOR);
 		}
 		break;
 	}
@@ -2223,6 +2424,8 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		gint cursor_row, cursor_col;
 		gint handled = TRUE;
 
+		d(g_print("%s: GDK_KEY_PRESS received, keyval: %d\n", __FUNCTION__, (int) e->key.keyval));
+
 		gtk_object_get(GTK_OBJECT(eti->selection),
 			       "cursor_row", &cursor_row,
 			       "cursor_col", &cursor_col,
@@ -2331,12 +2534,12 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		case GDK_ISO_Enter:
 		case GDK_3270_Enter:
 			if (eti_editing (eti)){
-				e_table_item_leave_edit (eti);
+				e_table_item_leave_edit_(eti);
 #if 0
 				ecell_view = eti->cell_views [eti->editing_col];
 				return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
 							       view_to_model_col(eti, eti->editing_col),
-							       eti->editing_col, eti->editing_row, E_CELL_EDITING);
+							       eti->editing_col, eti->editing_row, E_CELL_EDITING | E_CELL_CURSOR);
 #endif
 			}
 			gtk_signal_emit (GTK_OBJECT (eti), eti_signals [KEY_PRESS],
@@ -2389,7 +2592,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 					ecell_view = eti->cell_views [eti->editing_col];
 					return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
 								       view_to_model_col(eti, eti->editing_col),
-								       eti->editing_col, eti->editing_row, E_CELL_EDITING);
+								       eti->editing_col, eti->editing_row, E_CELL_EDITING | E_CELL_CURSOR);
 					if (!return_val)
 						e_selection_model_key_press(E_SELECTION_MODEL (eti->selection), (GdkEventKey *) e);
 				}
@@ -2403,6 +2606,8 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 	case GDK_KEY_RELEASE: {
 		gint cursor_row, cursor_col;
 
+		d(g_print("%s: GDK_KEY_RELEASE received, keyval: %d\n", __FUNCTION__, (int) e->key.keyval));
+
 		gtk_object_get(GTK_OBJECT(eti->selection),
 			       "cursor_row", &cursor_row,
 			       "cursor_col", &cursor_col,
@@ -2415,29 +2620,34 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			ecell_view = eti->cell_views [eti->editing_col];
 			return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
 						       view_to_model_col(eti, eti->editing_col),
-						       eti->editing_col, eti->editing_row, E_CELL_EDITING);
+						       eti->editing_col, eti->editing_row, E_CELL_EDITING | E_CELL_CURSOR);
 		}
 		break;
 	}
 
 	case GDK_LEAVE_NOTIFY:
+		d(leave = TRUE);
 	case GDK_ENTER_NOTIFY:
+		d(g_print("%s: %s received\n", __FUNCTION__, leave ? "GDK_LEAVE_NOTIFY" : "GDK_ENTER_NOTIFY"));
 		if (eti->tooltip->timer > 0)
 			gtk_timeout_remove (eti->tooltip->timer);
 		eti->tooltip->timer = 0;
 		break;
 
 	case GDK_FOCUS_CHANGE:
+		d(g_print("%s: GDK_FOCUS_CHANGE received, %s\n", __FUNCTION__, e->focus_change.in ? "in": "out"));
 		if (! e->focus_change.in) {
 			if (eti_editing (eti))
-				e_table_item_leave_edit (eti);
+				e_table_item_leave_edit_(eti);
 		}
 
 	default:
 		return_val = FALSE;
 	}
+	d(g_print("%s: returning: %s\n", __FUNCTION__, return_val?"true":"false"));
+
 	return return_val;
-	}
+}
 
 static void
 eti_class_init (GtkObjectClass *object_class)
@@ -2650,7 +2860,7 @@ eti_cursor_change (ESelectionModel *selection, int row, int col, ETableItem *eti
 	view_col = model_to_view_col(eti, col);
 	
 	if (view_row == -1 || view_col == -1) {
-		e_table_item_leave_edit (eti);
+		e_table_item_leave_edit_(eti);
 		return;
 	}
 
@@ -2664,7 +2874,7 @@ eti_cursor_change (ESelectionModel *selection, int row, int col, ETableItem *eti
 
 	e_canvas_item_grab_focus(GNOME_CANVAS_ITEM(eti), FALSE);
 	if (eti_editing(eti))
-		e_table_item_leave_edit (eti);
+		e_table_item_leave_edit_(eti);
 	gtk_signal_emit (GTK_OBJECT (eti), eti_signals [CURSOR_CHANGE],
 			 view_row);
 	eti->needs_redraw = TRUE;
@@ -2683,21 +2893,19 @@ eti_cursor_activated (ESelectionModel *selection, int row, int col, ETableItem *
 	view_row = model_to_view_row(eti, row);
 	view_col = model_to_view_col(eti, col);
 	
-	if (view_row == -1 || view_col == -1) {
-		e_table_item_leave_edit (eti);
-		return;
-	}
-
-	if (! e_table_model_has_change_pending (eti->table_model)) {
-		if (!eti->in_key_press) {
-			eti_show_cursor(eti, DOUBLE_CLICK_TIME + 10);
-		} else {
-			eti_show_cursor(eti, 0);
+	if (view_row != -1 && view_col != -1) {
+		if (! e_table_model_has_change_pending (eti->table_model)) {
+			if (!eti->in_key_press) {
+				eti_show_cursor(eti, DOUBLE_CLICK_TIME + 10);
+			} else {
+				eti_show_cursor(eti, 0);
+			}
+			eti_check_cursor_bounds (eti);
 		}
 	}
 
 	if (eti_editing(eti))
-		e_table_item_leave_edit (eti);
+		e_table_item_leave_edit_(eti);
 	gtk_signal_emit (GTK_OBJECT (eti), eti_signals [CURSOR_ACTIVATED],
 			 view_row);
 }
@@ -2728,8 +2936,10 @@ e_table_item_enter_edit (ETableItem *eti, int col, int row)
 	g_return_if_fail (eti != NULL);
 	g_return_if_fail (E_IS_TABLE_ITEM (eti));
 	
+	d(g_print("%s: %d, %d, eti_editing() = %s\n", __FUNCTION__, col, row, eti_editing(eti)?"true":"false"));
+
 	if (eti_editing (eti))
-		e_table_item_leave_edit(eti);
+		e_table_item_leave_edit_(eti);
 
 	eti->editing_col = col;
 	eti->editing_row = row;
@@ -2738,7 +2948,7 @@ e_table_item_enter_edit (ETableItem *eti, int col, int row)
 }
 
 /** 
- * e_table_item_leave_edit
+ * e_table_item_leave_edit_
  * @eti: %ETableItem which will stop being edited
  *
  * This routine stops the given %ETableItem from editing.
@@ -2752,6 +2962,8 @@ e_table_item_leave_edit (ETableItem *eti)
 	g_return_if_fail (eti != NULL);
 	g_return_if_fail (E_IS_TABLE_ITEM (eti));
 
+	d(g_print("%s: eti_editing() = %s\n", __FUNCTION__, eti_editing(eti)?"true":"false"));
+
 	if (!eti_editing (eti))
 		return;
 
