@@ -1,15 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-tree-model.c: a Tree Model
+/* 
+ * e-tree-model.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Chris Toshok (toshok@ximian.com)
- *   Chris Lahey  <clahey@ximian.com>
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Chris Toshok <toshok@ximian.com>
  *
- * Adapted from the gtree code and ETableModel.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * (C) 2000, 2001 Ximian, Inc.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 
 #include <stdio.h>
@@ -34,6 +46,7 @@ static GtkObjectClass *parent_class;
 
 enum {
 	PRE_CHANGE,
+	NO_CHANGE,
 	NODE_CHANGED,
 	NODE_DATA_CHANGED,
 	NODE_COL_CHANGED,
@@ -55,15 +68,23 @@ e_tree_model_class_init (GtkObjectClass *klass)
 	e_tree_model_signals [PRE_CHANGE] =
 		gtk_signal_new ("pre_change",
 				GTK_RUN_LAST,
-				klass->type,
+				E_OBJECT_CLASS_TYPE (klass),
 				GTK_SIGNAL_OFFSET (ETreeModelClass, pre_change),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
+	e_tree_model_signals [NO_CHANGE] =
+		gtk_signal_new ("no_change",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (klass),
+				GTK_SIGNAL_OFFSET (ETreeModelClass, no_change),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
+
 	e_tree_model_signals [NODE_CHANGED] =
 		gtk_signal_new ("node_changed",
 				GTK_RUN_LAST,
-				klass->type,
+				E_OBJECT_CLASS_TYPE (klass),
 				GTK_SIGNAL_OFFSET (ETreeModelClass, node_changed),
 				gtk_marshal_NONE__POINTER,
 				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
@@ -71,7 +92,7 @@ e_tree_model_class_init (GtkObjectClass *klass)
 	e_tree_model_signals [NODE_DATA_CHANGED] =
 		gtk_signal_new ("node_data_changed",
 				GTK_RUN_LAST,
-				klass->type,
+				E_OBJECT_CLASS_TYPE (klass),
 				GTK_SIGNAL_OFFSET (ETreeModelClass, node_data_changed),
 				gtk_marshal_NONE__POINTER,
 				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
@@ -79,7 +100,7 @@ e_tree_model_class_init (GtkObjectClass *klass)
 	e_tree_model_signals [NODE_COL_CHANGED] =
 		gtk_signal_new ("node_col_changed",
 				GTK_RUN_LAST,
-				klass->type,
+				E_OBJECT_CLASS_TYPE (klass),
 				GTK_SIGNAL_OFFSET (ETreeModelClass, node_col_changed),
 				gtk_marshal_NONE__POINTER_INT,
 				GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_INT);
@@ -87,7 +108,7 @@ e_tree_model_class_init (GtkObjectClass *klass)
 	e_tree_model_signals [NODE_INSERTED] =
 		gtk_signal_new ("node_inserted",
 				GTK_RUN_LAST,
-				klass->type,
+				E_OBJECT_CLASS_TYPE (klass),
 				GTK_SIGNAL_OFFSET (ETreeModelClass, node_inserted),
 				gtk_marshal_NONE__POINTER_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_POINTER);
@@ -95,12 +116,12 @@ e_tree_model_class_init (GtkObjectClass *klass)
 	e_tree_model_signals [NODE_REMOVED] =
 		gtk_signal_new ("node_removed",
 				GTK_RUN_LAST,
-				klass->type,
+				E_OBJECT_CLASS_TYPE (klass),
 				GTK_SIGNAL_OFFSET (ETreeModelClass, node_removed),
 				e_marshal_NONE__POINTER_POINTER_INT,
 				GTK_TYPE_NONE, 3, GTK_TYPE_POINTER, GTK_TYPE_POINTER, GTK_TYPE_INT);
 
-	gtk_object_class_add_signals (klass, e_tree_model_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (klass, e_tree_model_signals, LAST_SIGNAL);
 
 	tree_class->get_root             = NULL;
 
@@ -138,6 +159,7 @@ e_tree_model_class_init (GtkObjectClass *klass)
 	tree_class->value_to_string      = NULL;
 
 	tree_class->pre_change           = NULL;
+	tree_class->no_change            = NULL;
 	tree_class->node_changed         = NULL;
 	tree_class->node_data_changed    = NULL;
 	tree_class->node_col_changed     = NULL;
@@ -174,6 +196,25 @@ e_tree_model_pre_change  (ETreeModel *tree_model)
 			 e_tree_model_signals [PRE_CHANGE]);
 }
 
+/**
+ * e_tree_model_node_changed:
+ * @tree_model: 
+ * @node: 
+ * 
+ * 
+ * 
+ * Return value: 
+ **/
+void
+e_tree_model_no_change  (ETreeModel *tree_model)
+{
+	g_return_if_fail (tree_model != NULL);
+	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
+	
+	gtk_signal_emit (GTK_OBJECT (tree_model),
+			 e_tree_model_signals [NO_CHANGE]);
+}
+
 /**
  * e_tree_model_node_changed:
  * @tree_model: 
