@@ -1,12 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-tree-selection-model.c: a Selection Model
+/* 
+ * e-tree-selection-model.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Christopher James Lahey <clahey@ximian.com>
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 2000, 2001 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
@@ -56,7 +70,10 @@ struct ETreeSelectionModelPriv {
 
 	char *cursor_save_id;
 
+	int          frozen_count;
+
 	int          tree_model_pre_change_id;
+	int          tree_model_no_change_id;
 	int          tree_model_node_changed_id;
 	int          tree_model_node_data_changed_id;
 	int          tree_model_node_col_changed_id;
@@ -327,6 +344,18 @@ update_parents (ETreeSelectionModel *etsm, ETreePath path)
 
 /* Signal handlers */
 
+static void
+etsm_freeze (ETreeSelectionModel *etsm)
+{
+	etsm->priv->frozen_count ++;
+}
+
+static void
+etsm_unfreeze (ETreeSelectionModel *etsm)
+{
+	etsm->priv->frozen_count --;
+}
+
 static void
 etsm_pre_change (ETreeModel *etm, ETreeSelectionModel *etsm)
 {
@@ -338,6 +367,14 @@ etsm_pre_change (ETreeModel *etm, ETreeSelectionModel *etsm)
 	    etsm->priv->cursor_path) {
 		etsm->priv->cursor_save_id = e_tree_model_get_save_id (etm, etsm->priv->cursor_path);
 	}
+
+	etsm_freeze (etsm);
+}
+
+static void
+etsm_no_change (ETreeModel *etm, ETreeSelectionModel *etsm)
+{
+	etsm_unfreeze (etsm);
 }
 
 static void
@@ -370,6 +407,7 @@ etsm_node_changed (ETreeModel *etm, ETreePath node, ETreeSelectionModel *etsm)
 
 	g_free (etsm->priv->cursor_save_id);
 	etsm->priv->cursor_save_id = NULL;
+	etsm_unfreeze (etsm);
 }
 
 static void
@@ -377,6 +415,7 @@ etsm_node_data_changed (ETreeModel *etm, ETreePath node, ETreeSelectionModel *et
 {
 	g_free (etsm->priv->cursor_save_id);
 	etsm->priv->cursor_save_id = NULL;
+	etsm_unfreeze (etsm);
 }
 
 static void
@@ -384,6 +423,7 @@ etsm_node_col_changed (ETreeModel *etm, ETreePath node, int col, ETreeSelectionM
 {
 	g_free (etsm->priv->cursor_save_id);
 	etsm->priv->cursor_save_id = NULL;
+	etsm_unfreeze (etsm);
 }
 
 static void
@@ -440,6 +480,8 @@ add_model(ETreeSelectionModel *etsm, ETreeModel *model)
 	gtk_object_ref(GTK_OBJECT(priv->model));
 	priv->tree_model_pre_change_id        = gtk_signal_connect_after (GTK_OBJECT (priv->model), "pre_change",
 									  GTK_SIGNAL_FUNC (etsm_pre_change), etsm);
+	priv->tree_model_no_change_id         = gtk_signal_connect_after (GTK_OBJECT (priv->model), "no_change",
+									  GTK_SIGNAL_FUNC (etsm_no_change), etsm);
 	priv->tree_model_node_changed_id      = gtk_signal_connect_after (GTK_OBJECT (priv->model), "node_changed",
 									  GTK_SIGNAL_FUNC (etsm_node_changed), etsm);
 	priv->tree_model_node_data_changed_id = gtk_signal_connect_after (GTK_OBJECT (priv->model), "node_data_changed",
@@ -462,6 +504,8 @@ drop_model(ETreeSelectionModel *etsm)
 
 	gtk_signal_disconnect (GTK_OBJECT (priv->model),
 			       priv->tree_model_pre_change_id);
+	gtk_signal_disconnect (GTK_OBJECT (priv->model),
+			       priv->tree_model_no_change_id);
 	gtk_signal_disconnect (GTK_OBJECT (priv->model),
 			       priv->tree_model_node_changed_id);
 	gtk_signal_disconnect (GTK_OBJECT (priv->model),
@@ -477,6 +521,7 @@ drop_model(ETreeSelectionModel *etsm)
 	priv->model = NULL;
 
 	priv->tree_model_pre_change_id = 0;
+	priv->tree_model_no_change_id = 0;
 	priv->tree_model_node_changed_id = 0;
 	priv->tree_model_node_data_changed_id = 0;
 	priv->tree_model_node_col_changed_id = 0;
@@ -599,19 +644,18 @@ etsm_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 }
 
 static ETreeSelectionModelNode *
-etsm_recurse_is_path_selected (ESelectionModel *selection,
+etsm_recurse_is_path_selected (ETreeSelectionModel *etsm,
 			       ETreePath path,
 			       gboolean *is_selected)
 {
 	ETreeSelectionModelNode *selection_node;
-	ETreeSelectionModel *etsm = E_TREE_SELECTION_MODEL(selection);
 	ETreeSorted *ets = etsm->priv->ets;
 	ETreePath parent;
 
 	parent = e_tree_model_node_get_parent(E_TREE_MODEL(ets), path);
 
 	if (parent) {
-		selection_node = etsm_recurse_is_path_selected (selection, parent, is_selected);
+		selection_node = etsm_recurse_is_path_selected (etsm, parent, is_selected);
 		if (selection_node) {
 			int position = e_tree_sorted_orig_position(ets, path);
 			if (position < 0 || position >= selection_node->num_children) {
@@ -651,6 +695,21 @@ etsm_recurse_is_path_selected (ESelectionModel *selection,
 	}
 }
 
+static gboolean
+etsm_is_path_selected (ETreeSelectionModel *etsm,
+		       ETreePath path)
+{
+	ETreeSelectionModelNode *selection_node;
+	gboolean ret_val;
+
+	selection_node = etsm_recurse_is_path_selected (etsm, path, &ret_val);
+
+	if (selection_node)
+		ret_val = selection_node->selected;
+
+	return ret_val;
+}
+
 /** 
  * e_selection_model_is_row_selected
  * @selection: #ESelectionModel to check
@@ -666,22 +725,13 @@ etsm_is_row_selected (ESelectionModel *selection,
 {
 	ETreeSelectionModel *etsm = E_TREE_SELECTION_MODEL(selection);
 	ETreePath path;
-	ETreeSelectionModelNode *selection_node;
-
-	gboolean ret_val;
 
 	g_return_val_if_fail(row < e_table_model_row_count(E_TABLE_MODEL(etsm->priv->etta)), FALSE);
 	g_return_val_if_fail(row >= 0, FALSE);
-	g_return_val_if_fail(selection != NULL, FALSE);
+	g_return_val_if_fail(etsm != NULL, FALSE);
 
 	path = e_tree_table_adapter_node_at_row(etsm->priv->etta, row);
-
-	selection_node = etsm_recurse_is_path_selected (selection, path, &ret_val);
-
-	if (selection_node)
-		ret_val = selection_node->selected;
-
-	return ret_val;
+	return etsm_is_path_selected (etsm, path);
 }
 
 
@@ -803,6 +853,9 @@ etsm_selected_count (ESelectionModel *selection)
 		ETreePath model_root;
 		model_root = e_tree_model_get_root(etsm->priv->model);
 		etsm_selected_count_recurse(etsm, etsm->priv->root, model_root, &count);
+		if (!e_tree_table_adapter_root_node_is_visible (etsm->priv->etta) && etsm_is_path_selected (etsm, e_tree_model_get_root(etsm->priv->model))) {
+			count --;
+		}
 	}
 	return count;
 }
@@ -1172,6 +1225,7 @@ e_tree_selection_model_init (ETreeSelectionModel *etsm)
 
 
 	priv->tree_model_pre_change_id          = 0;
+	priv->tree_model_no_change_id          = 0;
 	priv->tree_model_node_changed_id        = 0;
 	priv->tree_model_node_data_changed_id   = 0;
 	priv->tree_model_node_col_changed_id    = 0;
