@@ -1,23 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
- * e-xml-utils.c
- * Copyright (C) 2000  Helix Code, Inc.
- * Author: Chris Lahey <clahey@helixcode.com>
+ * e-util.c
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <glib.h>
@@ -195,7 +196,37 @@ e_write_file(const char *filename, const char *data, int flags)
 			}
 		}
 	}
-	close(fd);
+	if (close(fd) != 0) {
+		return errno;
+	}
+	return 0;
+}
+
+gint
+e_write_file_mkstemp(char *filename, const char *data)
+{
+	int fd;
+	int length = strlen(data);
+	int bytes;
+	fd = mkstemp (filename);
+	if (fd == -1)
+		return errno;
+	while (length > 0) {
+		bytes = write(fd, data, length);
+		if (bytes > 0) {
+			length -= bytes;
+			data += bytes;
+		} else {
+			if (errno != EINTR && errno != EAGAIN) {
+				int save_errno = errno;
+				close(fd);
+				return save_errno;
+			}
+		}
+	}
+	if (close(fd) != 0) {
+		return errno;
+	}
 	return 0;
 }
 
@@ -704,10 +735,31 @@ e_marshal_INT__POINTER_POINTER_POINTER_POINTER_POINTER (GtkObject *object,
 				GTK_VALUE_POINTER (args[1]), 
 				GTK_VALUE_POINTER (args[2]),
 				GTK_VALUE_POINTER (args[3]),
-				GTK_VALUE_POINTER (args[3]),
+				GTK_VALUE_POINTER (args[4]),
 				func_data);
 }
 
+typedef void (*GtkSignal_NONE__POINTER_POINTER_POINTER_BOOL) (GtkObject *,
+							      gpointer, gpointer, gpointer, gboolean,
+							      gpointer user_data);
+
+void
+e_marshal_NONE__POINTER_POINTER_POINTER_BOOL (GtkObject *object,
+					      GtkSignalFunc func,
+					      gpointer func_data,
+					      GtkArg *args)
+{
+	GtkSignal_NONE__POINTER_POINTER_POINTER_BOOL rfunc;
+
+	rfunc = (GtkSignal_NONE__POINTER_POINTER_POINTER_BOOL) func;
+
+	(*rfunc) (object,
+		  GTK_VALUE_POINTER (args[0]),
+		  GTK_VALUE_POINTER (args[1]), 
+		  GTK_VALUE_POINTER (args[2]),
+		  GTK_VALUE_BOOL (args[3]),
+		  func_data);
+}
 
 void
 e_marshal_NONE__POINTER_INT_INT_INT (GtkObject       *object,
@@ -858,11 +910,9 @@ e_format_number (gint number)
 	int divider;
 	char *value;
 	char *value_iterator;
-	int initial_grouping;
 
 	locality = localeconv();
 	grouping = locality->grouping;
-	initial_grouping = *grouping;
 	while (number) {
 		char *group;
 		switch (*grouping) {
@@ -871,8 +921,8 @@ e_format_number (gint number)
 			grouping++;
 		case 0:
 			divider = epow10(last_count);
-			if((!list && (number/divider) > 0) || number > divider) {
-				group = g_strdup_printf("%0*d", initial_grouping, number % divider);
+			if (number >= divider) {
+				group = g_strdup_printf("%0*d", last_count, number % divider);
 			} else {
 				group = g_strdup_printf("%d", number % divider);
 			}
@@ -1402,3 +1452,56 @@ e_ascii_dtostr (gchar       *buffer,
 
 	return buffer;
 }
+
+gchar *
+e_strdup_append_strings (gchar *first_string, ...)
+{
+	gchar *buffer;
+	gchar *current;
+	gint length;
+	va_list args1;
+	va_list args2;
+	char *v_string;
+	int v_int;
+
+	va_start (args1, first_string);
+	G_VA_COPY (args2, args1);
+
+	length = 0;
+
+	v_string = first_string;
+	while (v_string) {
+		v_int = va_arg (args1, int);
+		if (v_int >= 0)
+			length += v_int;
+		else
+			length += strlen (v_string);
+		v_string = va_arg (args1, char *);
+	}
+
+	buffer  = g_new (char, length + 1);
+	current = buffer;
+
+	v_string = first_string;
+	while (v_string) {
+		v_int = va_arg (args2, int);
+		if (v_int < 0) {
+			int i;
+			for (i = 0; v_string[i]; i++) {
+				*(current++) = v_string[i];
+			}
+		} else {
+			int i;
+			for (i = 0; v_string[i] && i < v_int; i++) {
+				*(current++) = v_string[i];
+			}
+		}
+		v_string = va_arg (args2, char *);
+	}
+	*(current++) = 0;
+
+	va_end (args1);
+	va_end (args2);
+
+	return buffer;
+}
